#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "ball.h"
#include "platform.h"
#include <QKeyEvent>
#include <QMainWindow>
#include <QWidget>
#include <qevent.h>

QT_BEGIN_NAMESPACE
namespace Ui {
    class MainWindow;
}
QT_END_NAMESPACE

class MainWindow : public QMainWindow {
    Q_OBJECT

  public:
    MainWindow(QWidget* parent = nullptr);
    ~MainWindow();

    bool leftArrowPressed;
    bool rightArrowPressed;

  private slots:
    /**
    * @brief Handles the event when a ball falls through the bottom.
    * @param ball The ball that fell through.
    */
    void ballFell(Ball* ball);

    /**
    * @brief Handles the event when a ball is caught by the platform.
    * @param ball The ball that was caught.
    */
    void ballCaught(Ball* ball);

    /**
    * @brief Handles the ball spawn timer event.
    */
    void onBallSpawn();

  protected:
    /**
    * @brief Paint event handler to render the game objects.
    * @param event Paint event.
    */
    virtual void paintEvent(QPaintEvent*) override;

    /**
    * @brief Deletes a given ball from the game.
    * @param ball The ball to be deleted.
    */
    void deleteBall(Ball* ball);

    /**
    * @brief Spawns a new ball in the game.
    */
    void spawnBall();

    void keyPressEvent(QKeyEvent* event) override;
    void keyReleaseEvent(QKeyEvent* event) override;

  private:
    int m_score;
    float m_spawnInterval;
    QTimer m_spawnTimer;
    Ui::MainWindow* ui;
    QList<std::shared_ptr<Ball>> balls;
    std::shared_ptr<Platform> platform;
};
#endif // MAINWINDOW_H
